//@author:  Catweasel
//@help: basic vertex based lightning with point light
//@tags: shading, blinn
//@credits:vvvv group

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (DX9)
float4x4 tWV: WORLDVIEW;
float4x4 tWVP: WORLDVIEWPROJECTION;
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (DX9)

float3 SpotLightPos : POSITION <string UIName = "Lamp Posistion";> = {-1.0f, 1.0f, 0.0f};
float PCFSize <float UIMin = 0.100;string UIName = "PCF Filter Width";> = 0.01;
float SpotLightCone <string UIName = "Cone Angle";float uimin=0.0;> = 60.0;
////////////////////////////////////////////////////////////////////////////////
float4x4 LampViewXf;
float4x4 LampProjXf;


Texture2D shadowTex <string uiname="shadowTexture";>;

SamplerComparisonState PCF_Sampler  : IMMUTABLE  { 
	Filter = COMPARISON_MIN_MAG_MIP_LINEAR;
	AddressU = Clamp;
	AddressV = Clamp;
	ComparisonFunc = LESS;
};

/////////////////////////////////////////////////////////////////////
struct vs2ps
{
    float4 PosWVP: SV_POSITION;

	float4 LP     : TEXCOORD1;
};
/////////////////////////////////////////////////////////////////////
vs2ps VS(
    float4 PosO: POSITION,
    float3 NormO: NORMAL,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //distance from light to vertex
    float4 PosW = mul(PosO, tW);
////// shadow light pos/////////////////////////////////////////////
	float4x4 ShadowViewProjXf = mul(LampViewXf,LampProjXf);    // light viewprojection
    float4 Pl = mul(PosW,ShadowViewProjXf);  // "P" in light coords
    Out.LP = Pl;
//////////////////////////////////////////////////////////////////
    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

//////////////////////////////////////////////////////////////////
float2 poissonDisk[16] = {float2( -0.94201624, -0.39906216 ), float2 ( 0.94558609, -0.76890725 ), float2 ( -0.094184101, -0.92938870 ),float2 ( 0.34495938, 0.29387760 ), float2 ( -0.91588581, 0.45771432 ), float2 ( -0.81544232, -0.87912464 ),float2 ( -0.38277543, 0.27676845 ), float2 ( 0.97484398, 0.75648379 ), float2 ( 0.44323325, -0.97511554 ),float2 ( 0.53742981, -0.47373420 ), float2 ( -0.26496911, -0.41893023 ), float2 ( 0.79197514, 0.19090188 ),float2 ( -0.24188840, 0.99706507 ), float2 ( -0.81409955, 0.91437590 ), float2 ( 0.19984126, 0.78641367 ),float2 ( 0.14383161, -0.14100790 ) };
#define PCFH (7.5)
#define PCFT (256)
float bias;
float ShadowDensity;
float Boost=1;




///////////////////////////////////////////////////////////////////////
// Utility function for pixel shaders to use this shadow map
float shadow_calc(float4 LP,	// current shaded point in light-projected coordinates
	Texture2D ShadowMapTex)
{

    float totalShad=0;
	int i;
	float offSize = PCFSize / 16;
	for (i = 0; i<= PCFH; i += 1) 
	{
//		float2 offset = float2(offSize*offsets[i]);
		float2 offset = float2(offSize*poissonDisk[i]);
		float2 nuv = float2(.5,-.5)*(LP.xy+offset)/LP.w + float2(.5,.5);
		float shadMapDepth = ShadowMapTex.SampleCmpLevelZero(PCF_Sampler,nuv,LP.z/LP.w-bias).x;
		
		float shad = 1-(shadMapDepth<LP.z);
		//totalShad += shad;
		totalShad +=shadMapDepth;
	}
	return totalShad/14;
}
///////////////////////////////////////////////////////////////////////
float4 PS(vs2ps In): SV_Target
{
	float CosSpotAng = cos(radians(SpotLightCone));
	float dl = normalize(In.LP.xyz).z;
    dl = max((float)0,((dl-CosSpotAng)/(((float)1.0)-CosSpotAng)));
	float shadowed = clamp(shadow_calc(In.LP,shadowTex)+ShadowDensity,0,1);	
    return shadowed*dl*Boost ;
}


// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------
technique10 JustShadow
{
	pass P0
	{
		SetGeometryShader( 0 );
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
		SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}


